import unittest
from conf.testing import czifile
from model_server.accessors import CziImageFileAccessor
from model_server.models import DummyImageToImageModel, CouldNotLoadModelError

class TestCziImageFileAccess(unittest.TestCase):
    def setUp(self) -> None:
        self.cf = CziImageFileAccessor(czifile['path'])

    def test_instantiate_model(self):
        model = DummyImageToImageModel(params=None)
        self.assertTrue(model.loaded)

    def test_instantiate_model_with_nondefault_kwarg(self):
        model = DummyImageToImageModel(autoload=False)
        self.assertFalse(model.autoload, 'Could not override autoload flag in subclass of Model.')

    def test_raise_error_if_cannot_load_model(self):
        class UnloadableDummyImageToImageModel(DummyImageToImageModel):
            def load(self):
                return False

        with self.assertRaises(CouldNotLoadModelError):
            mi = UnloadableDummyImageToImageModel()

    def test_czifile_is_correct_shape(self):
        model = DummyImageToImageModel()
        img, _ = model.infer(self.cf)

        w = czifile['w']
        h = czifile['h']

        self.assertEqual(
            img.shape,
            (h, w, 1, 1),
            'Inferred image is not the expected shape'
        )

        self.assertEqual(
            img.data[int(w/2), int(h/2)],
            255,
            'Middle pixel is not white as expected'
        )

        self.assertEqual(
            img.data[0, 0],
            0,
            'First pixel is not black as expected'
        )